package org.apache.turbine.services.db;

/*
 * Copyright (c) 1997-2000 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * 4. The names "Apache JServ", "Apache JServ Servlet Engine", "Turbine",
 *    "Apache Turbine", "Turbine Project", "Apache Turbine Project" and
 *    "Java Apache Project" must not be used to endorse or promote products
 *    derived from this software without prior written permission.
 *
 * 5. Products derived from this software may not be called "Apache JServ"
 *    nor may "Apache" nor "Apache JServ" appear in their names without
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Group. For more information
 * on the Java Apache Project and the Apache JServ Servlet Engine project,
 * please see <http://java.apache.org/>.
 *
 */

// Java Core Classes
import java.util.Hashtable;
import java.util.Enumeration;
import java.util.Properties;

// Turbine Utility Classes
import org.apache.turbine.util.Log;
import org.apache.turbine.util.db.map.DatabaseMap;
import org.apache.turbine.util.db.map.TableMap;
import org.apache.turbine.util.db.map.MapBuilder;
import org.apache.turbine.services.BaseService;
import org.apache.turbine.services.InitializationException;
import org.apache.turbine.services.ServiceBroker;
import org.apache.turbine.util.db.pool.*;
import org.apache.turbine.util.db.adapter.DB;
import org.apache.turbine.services.resources.TurbineResources;

/**
 * This class is the single point of access to connection pools.<p>
 *
 * To retrieve a Connection object from this singleton do either:<p>
 *
 * 1) Get a connection using default values specified in a
 * properties file.<br>
 * DBConnection db = PoolBrokerService.getInstance().getConnection();<br>
 * Connection con = db.getConnection();
 * <p>
 * 2) Get a connection using the given parameters.<br>
 * DBConnection db = PoolBrokerService.getInstance().getConnection(driver, url,
 *                                                        username,
 *                                                        password)<br>
 * Connection con = db.getConnection();
 * <p>
 * Call PoolBrokerService.getInstance().releaseConnection(db) when you are
 * done.
 *
 * @author <a href="mailto:frank.kim@clearink.com">Frank Y. Kim</a>
 * @author <a href="mailto:bmclaugh@algx.net">Brett McLaughlin</a>
 * @author <a href="mailto:greg@shwoop.com">Greg Ritter</a>
 * @author <a href="mailto:dlr@finemaltcoding.com">Daniel Rall</a>
 * @author <a href="mailto:magnus@handtolvur.is">Magns r Torfason</a>
 * @author <a href="mailto:jvanzyl@periapt.com">Jason van Zyl</a>
 * @version $Id: PoolBrokerService.java,v 1.8 2000/11/27 00:09:25 jvanzyl Exp $
 */
public class PoolBrokerService extends BaseService
{
    /**
     * Name of this <code>Service</code>.
     */
    public static final String SERVICE_NAME = "PoolBrokerService";

    /**
     * Name of the default map builder and map.
     * TODO: Move this into its own Turbine Service.
     */
    public static final String DEFAULT = "default";

    /**
     * The single instance of this class.
     */
    private static PoolBrokerService instance = null;

    /**
     * The various connection pools this broker contains.  Keyed by
     * database URL.
     */
    protected Hashtable pools = null;

    /**
     * The DatabaseMap which contains the tables in this DB.
     * TODO: Move dbMaps into its own Turbine Service.
     */
    protected Hashtable dbMaps = null;

    /**
     * An instance of the Monitor class, to monitor the use of this class.
     */
    protected Monitor monitor = null;

    /**
     * Consumers should always employ the <code>getInstance()</code>
     * method to get the single instance of this class.
     *
     * Note that this ctor should probably be switched back to private when
     * its deprecated extension in the
     * <code>org.apache.turbine.util.db.pool</code> package is removed.
     */
    public PoolBrokerService()
    {
        super();

        pools = new Hashtable();
        // TODO: Move dbMaps into its own Turbine Service.
        dbMaps = new Hashtable();
        monitor = new Monitor();
        monitor.start();

    }

    /**
     * This thread-safe method registers a new pool using the given parameters.
     *
     * @param name The name of the pool to register.
     * @param driver The fully-qualified name of the JDBC driver to use.
     * @param url The URL of the database to use.
     * @param username The name of the database user.
     * @param password The password of the database user.
     * @exception Exception A generic exception.
     */
    public void registerPool( String name,
                              String driver,
                              String url,
                              String username,
                              String password )
        throws Exception
    {

        // Quick (non-sync) check for the pool we want.
        if ( !pools.containsKey(name) )
        {
            // Pool not there...
            synchronized ( pools )
            {
                // ... sync and look again to avoid race collisions.
                if ( !pools.containsKey(name) )
                {
                    // Still not there.  Create and add.
                    ConnectionPool pool = 
                        new ConnectionPool(driver, url, username, password);
                    pools.put( name, pool );
                }
            }
        }
    }


    /**
     * This method returns a pool with the specified name.  The pool must 
     * either have been registered with the 
     * {@link registerConnection() registerConnection()} methd, or be 
     * specified in the TurbineResources properties file using the following 
     * syntax:
     *
     * <pre>
     * database.[name].driver
     * database.[name].url
     * database.[name].username
     * database.[name].password
     * </pre>
     *
     * @param name The name of the pool to get.
     * @return     The requested pool.
     *
     * @exception Exception A generic exception.
     */
    public ConnectionPool getPool(String name)
        throws Exception
    {
        ConnectionPool pool = (ConnectionPool) pools.get(name);

        // If the pool is not in the Hashtable, we must register it.
        if ( pool == null )
        {
            registerPool( name,
                          getDatabaseProperty(name, "driver"), 
                          getDatabaseProperty(name, "url"), 
                          getDatabaseProperty(name, "username"), 
                          getDatabaseProperty(name, "password") );
            pool = (ConnectionPool) pools.get(name);
        }

        return pool;
    }

    /**
     * Returns the specified property of the given database, or the empty 
     * string if no value is set for the property.
     *
     * @param db   The name of the database whose property to get.
     * @param prop The name of the property to get.
     * @return     The property's value.
     */
    private static final String getDatabaseProperty(String db, String prop)
    {
        return TurbineResources.getString
            ( new StringBuffer("database.")
                .append(db)
                .append('.')
                .append(prop)
                .toString(), "" );
    }

    /**
     * This method returns the default pool.
     *
     * @return The default pool.
     * @exception Exception A generic exception.
     */
    public ConnectionPool getPool()
        throws Exception
    {
        return getPool(DEFAULT);
    }


    /**
     * This method returns a DBConnection from the pool with the
     * specified name.  The pool must either have been registered
     * with the {@link registerConnection() registerConnection()}
     * methd, or be specified in the property file using the
     * following syntax:
     *
     * <pre>
     * database.[name].driver
     * database.[name].url
     * database.[name].username
     * database.[name].password
     * </pre>
     *
     * @param name The name of the pool to get a connection from.
     * @return     The requested connection.
     *
     * @exception Exception A generic exception.
     */
    public DBConnection getConnection(String name)
        throws Exception
    {
        // The getPool method ensures the validity of the returned pool.
        return getPool(name).getConnection();
    }

    /**
     * This method returns a DBConnection using the default values
     * specified in a properties file.
     *
     * @return The requested connection.
     * @exception Exception A generic exception.
     */
    public DBConnection getConnection()
        throws Exception
    {
        return getConnection( DEFAULT );
    }

    /**
     * This method returns a DBConnecton using the given parameters.
     *
     * @param driver The fully-qualified name of the JDBC driver to use.
     * @param url The URL of the database from which the connection is
     * desired.
     * @param username The name of the database user.
     * @param password The password of the database user.
     * @return A DBConnection.
     * @deprecated Database parameters should not be specified each
     * time a DBConnection is fetched from the service.
     * @exception Exception A generic exception.
     */
    public DBConnection getConnection(String driver,
                                      String url,
                                      String username,
                                      String password)
        throws Exception
    {
        ConnectionPool pool = null;
        url = url.trim();

        // Quick (non-sync) check for the pool we want.
        // NOTE: Here we must not call getPool(), since the pool
        // is almost certainly not defined in the properties file
        pool = (ConnectionPool) pools.get(url + username);
        if ( pool == null )
        {
            registerPool(url + username, driver,  url, username, password);
            pool = (ConnectionPool) pools.get(url + username);
        }

        return pool.getConnection();
    }

    /**
     * Returns the default database map information.
     * TODO: Move this into its own Turbine Service.
     *
     * @return A DatabaseMap.
     * @exception Exception A generic exception.
     */
    public DatabaseMap getDatabaseMap()
        throws Exception
    {
        return getDatabaseMap(DEFAULT);
    }

    /**
     * Returns the database map information. Name relates to the name
     * of the connection pool to associate with the map.
     * TODO: Move this into its own Turbine Service.
     *
     * @param name The name of the <code>DatabaseMap</code> to
     * retrieve.
     * @return The named <code>DatabaseMap</code>.
     * @exception Exception A generic exception.
     */
    public DatabaseMap getDatabaseMap(String name)
        throws Exception
    {
        if ( name == null )
        {
            throw new Exception ("DatabaseMap name was null!");
        }

        // Quick (non-sync) check for the map we want.
        DatabaseMap map = (DatabaseMap)dbMaps.get(name);
        if ( map == null )
        {
            // Map not there...
            synchronized( dbMaps )
            {
                // ... sync and look again to avoid race condition.
                map = (DatabaseMap)dbMaps.get(name);
                if ( map == null )
                {
                    // Still not there.  Create and add.
                    map = new DatabaseMap(name);
                    
                    /*
                     * Here we are making the Id Broker
                     * available to all applications by
                     * placing the mappings for the Id Broker
                     * in all database maps.
                     *
                     * So each database map that is requested
                     * will have the information below placed
                     * into it: each database map will know
                     * what the Id Table is, and its
                     * structure.
                     */
                    map.setIdTable("ID_TABLE");
                    TableMap tMap = map.getIdTable();
                    tMap.addPrimaryKey("ID_TABLE_ID", new Integer(0));
                    tMap.addColumn("TABLE_NAME", new String(""));
                    tMap.addColumn("NEXT_ID", new Integer(0));
                    tMap.addColumn("QUANTITY", new Integer(0));
                    
                    dbMaps.put(name, map);
                }
            }
        }

        return map;
    }

    /**
     * Adds a new database map.
     *
     * @param name Name of new DatabaseMap.
     * @deprecated A database map is added automatically by getDatabaseMap().
     */
    public void addDatabaseMap(String name)
    {
        addDatabaseMap( new DatabaseMap(name) );
    }

    /**
     * Put a DatabaseMap into the cache of maps it is keyed off the
     * name in the DatabaseMap.
     * TODO: Move this into its own Turbine Service.
     *
     * @param map A DatabaseMap.
     * @deprecated A database map is added automatically by getDatabaseMap().
     */
    public void addDatabaseMap(DatabaseMap map)
    {
        synchronized( dbMaps )
        {
            dbMaps.put (map.getName(), map);
        }
    }

    /**
     * Returns the database specific class.
     * TODO: Move this into its own Turbine Service.
     *
     * @return A DB.
     * @exception Exception A generic exception.
     */
    public DB getDB()
        throws Exception
    {
        return getDB(DEFAULT);
    }

    /**
     * Returns the requested database adapter.
     *
     * @param name Name of database adapter to retrieve.
     * @return     The requested database adapter.
     *
     * @exception Exception A generic exception.
     */
    public DB getDB(String name)
        throws Exception
    {
        return getPool(name).getDB();
    }

    /**
     * The method through which this class is accessed.
     *
     * @return A <code>PoolBrokerService</code>, the single instance
     *         of this class.
     */
    public static PoolBrokerService getInstance()
    {
        if (instance == null)
        {
            synchronized (PoolBrokerService.class)
            {
                if (instance == null)
                {
                    instance = new PoolBrokerService();
                }
            }
        }
        return instance;
    }

    /**
     * Release a connection back to the database pool.
     *
     * @param dbconn The connection to release.
     *
     * @exception Exception A generic exception.
     */
    public void releaseConnection(DBConnection dbconn)
        throws Exception
    {
        if ( dbconn != null )
        {
            ConnectionPool pool = dbconn.getPool();
            if ( pools.containsValue( pool ) )
            {
                pool.releaseConnection( dbconn );
            }
        }
    }

    /**
     * Releases the database connections for all pools on service shutdown.
     */
    public synchronized void shutdown()
    {
        if ( pools != null )
        {
            // Release connections for each pool.
            Enumeration poolEnum = pools.elements();
            while ( poolEnum.hasMoreElements() )
            {
                try
                {
                    ((ConnectionPool)poolEnum.nextElement()).shutdown();
                }
                catch (Exception ignored)
                {
                    // Unlikely.
                }
            }
        }
    }

    /**
     * This inner class monitors the <code>PoolBrokerService</code>.
     */
    protected class Monitor extends Thread
    {
        public void run()
        {
            int logInterval = TurbineResources.getInt("database.logInterval",0);
            StringBuffer buf = new StringBuffer();
            while (logInterval > 0)
            {
                // Loop through all pools and log.
                Enumeration poolKeys = pools.keys();
                while ( poolKeys.hasMoreElements() )
                {
                    String poolName = (String) poolKeys.nextElement();
                    ConnectionPool pool = (ConnectionPool) pools.get(poolName);
                    buf.setLength(0);
                    buf.append(poolName).append(" (in + out = total): ")
                        .append(pool.getNbrAvailable()).append(" + ")
                        .append(pool.getNbrCheckedOut()).append(" = ")
                        .append(pool.getTotalCount());
                    System.out.println(buf.toString());
                }

                // Wait for a bit.
                try
                {
                    Thread.sleep(logInterval);
                }
                catch (InterruptedException ignored)
                {
                    // Don't care.
                }
            }
        }
    }
}
