package org.apache.turbine.services.velocity;

/*
 * Copyright (c) 1997-2000 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * 4. The names "Apache JServ", "Apache JServ Servlet Engine", "Turbine",
 *    "Apache Turbine", "Turbine Project", "Apache Turbine Project" and
 *    "Java Apache Project" must not be used to endorse or promote products
 *    derived from this software without prior written permission.
 *
 * 5. Products derived from this software may not be called "Apache JServ"
 *    nor may "Apache" nor "Apache JServ" appear in their names without
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Group. For more information
 * on the Java Apache Project and the Apache JServ Servlet Engine project,
 * please see <http://java.apache.org/>.
 *
 */

// Java Stuff
import java.util.*;
import java.io.*;

// ServletConfig
import javax.servlet.*;

// Velocity Stuff
import org.apache.velocity.Context;
import org.apache.velocity.Template;
import org.apache.velocity.runtime.Runtime;
import org.apache.velocity.io.*;

// Turbine Stuff
import org.apache.turbine.services.*;
import org.apache.turbine.services.resources.*;

import org.apache.turbine.util.ImageURI;
import org.apache.turbine.util.Log;
import org.apache.turbine.util.RunData;
import org.apache.turbine.util.ServletUtils;

import org.apache.turbine.util.template.*;
import org.apache.turbine.modules.*;

/**
 * This is a Service that can process Velocity templates from within a
 * Turbine Screen.  Here's an example of how you might use it from a
 * screen:<br>
 *
 * <code>
 * Context context = TurbineVelocity.getContext(data);<br>
 * context.put("message", "Hello from Turbine!");<br>
 * String results = TurbineVelocity.handleRequest(context,"helloWorld.vm");<br>
 * data.getPage().getBody().addElement(results);<br>
 * </code>
 *
 * @author <a href="mailto:mbryson@mont.mindspring.com">Dave Bryson</a>
 * @author <a href="mailto:krzewski@e-point.pl">Rafal Krzewski</a>
 * @version $Id: TurbineVelocityService.java,v 1.10 2000/11/25 22:25:11 jvanzyl Exp $
 */
public class TurbineVelocityService
    extends TurbineBaseService
    implements VelocityService
{
    
    private String encoding;

    /**
     * Constructor.
     *
     * @exception Exception, a generic exception.
     */
    public TurbineVelocityService()
        throws Exception
    {
    }

    /**
     * Called during Turbine.init()
     *
     * @param config A ServletConfig.
     */
    public void init( ServletConfig config )
    {
        try
        {
            org.apache.turbine.util.Log.note ( "VelocityService early init()....starting!");
            initVelocity(config);
            org.apache.turbine.util.Log.note ( "VelocityService early init()....finished!");
        }
        catch (Exception e)
        {
            org.apache.turbine.util.Log.error ( "Cannot initialize TurbineVelocityService!" );
            org.apache.turbine.util.Log.error (e);
        }
        // we don't call setInit(true) yet, because we want init() to be called also
    }


    /**
     * Create an empty Context object.
     *
     * @return An empty Context object.
     */
    public Context getContext()
    {
        return new Context();
    }

    /**
     * Create a Context from the RunData object.  Adds a pointer to
     * the RunData object to the WC so that RunData is available in
     * the templates.
     *
     * @param data The Turbine RunData object.
     * @return A clone of the WebContext needed by Velocity.
     */
    public Context getContext(RunData data)
    {
        // Attempt to get it from the data first.  If it doesn't
        // exist, create it and then stuff it into the data.
        Context context = (Context)data.getTemplateInfo()
            .getTemplateContext(VelocityService.CONTEXT);
        if (context == null)
        {
            context = getContext();
            context.put ( "data", data );
            context.put ( "link", new TemplateLink(data) );
            context.put ( "page", new TemplatePageAttributes(data) );
            context.put ( "image", new ImageURI(data) );
            // ctxt.put ( "formatter", new VelocityFormatter(newWC) );
            data.getTemplateInfo()
                .setTemplateContext(VelocityService.CONTEXT, context);
        }
        return context;
    }
    
    /**
     * Return a template from Velocity.
     *
     * @param filename A String with the name of the template.
     * @return A Template.
     * @exception NotFoundException.
     */
    public Template getTemplate( String name )
     throws Exception
    {
        return Runtime.getTemplate(name);
    } 

    /**
     * Process the request and fill in the template with the values
     * you set in the Context.
     *
     * @param wc A Context.
     * @param filename A String with the filename of the template.
     * @return The process template as a String.
     * @exception Exception, a generic exception.
     */
    public String handleRequest(Context context, String filename)
        throws Exception
    {
        String results = null;
        ByteArrayOutputStream bytes = null;
        try
        {
            bytes = new ByteArrayOutputStream();
            handleRequest(context, filename, bytes);
            results = bytes.toString(encoding);
        } 
        catch(Exception e) 
        {
            org.apache.turbine.util.Log.error("An error occurred while rendering template "+filename+":\n"+e.getMessage());
            org.apache.turbine.util.Log.error(e);
        }
        finally
        {
            if (bytes != null) bytes.close();
        }
        return results;
    }

    /**
     * Process the request and fill in the template with the values
     * you set in the Context. 
     *
     * @param context A Context.
     * @param filename A String with the filename of the template.
     * @param out A OutputStream where we will write the process template as
     * a String.
     * @exception Exception, a generic exception.
     */
    public void handleRequest(Context context,
                              String filename,
                              OutputStream out)
        throws Exception
    {
        BufferedWriter writer = null;
        try
        {
            writer = new BufferedWriter(new OutputStreamWriter(out));
            Template template = getTemplate(filename);
            template.merge( context, writer );
        }
        catch(Exception e) 
        {
            org.apache.turbine.util.Log.error("An error occurred while rendering template "+filename+":\n"+e.getMessage());
            org.apache.turbine.util.Log.error(e);
        }
        finally
        {
            if (writer != null)
            {
                writer.flush();
                writer.close();
            }
        }
    }

    /**
     * This method initialized the Velocity Runtime which will be used to
     * load, access, and manage the templates. The Runtime first tries to
     * locate the Velocity properties file in the location specified
     * by the 'services.TurbineVelocityService.properties' key in the
     * TurbineResource file.  If the key is not set, it will search
     * the classpath looking for the file.
     *
     * @param config A ServletConfig.
     * @exception InitException.
     */
    private void initVelocity( ServletConfig config )
    {
        Properties props = getProperties();
        String vProperties = props.getProperty("properties",null);
        String vTemplates = props.getProperty("templates",null);
        String vLog = props.getProperty("log",null);

        //debug
        org.apache.turbine.util.Log.note("VelocityService properties:");
        Enumeration keys = props.propertyNames();
        while(keys.hasMoreElements())  {
            String key = (String)keys.nextElement();
            org.apache.turbine.util.Log.note(key+":"+props.getProperty(key));
        }
        //debug

        // If possible, transform paths to be webapp root relative.
        vProperties = ServletUtils.expandRelative(config,
                                                  vProperties);
        
        //debug
        Log.note("VelocityService is using "+vProperties);
        //debug

        if(vTemplates == null)
        {
            // Templates path is not specified in TurbineResources.properties
            //  fallback to Velocity default behaviour.
            return;
        }

        try
        {
            // If possible, transform paths to be webapp root
            // relative.
            vLog = ServletUtils.expandRelative(config,
                                               vLog);
            vTemplates = ServletUtils.expandRelative(config,
                                                     vTemplates);
            
            Runtime.setProperties(vProperties);
            encoding = Runtime.getString(Runtime.TEMPLATE_ENCODING);

            // Store the converted path in service properties for
            // Turbine based providers.
            props.put("log", vLog);
            // override the value set in velocity properties file.
            Runtime.setProperty(Runtime.RUNTIME_LOG, vLog);
            
            props.put("templates", vTemplates);
            // override the value set in velocity properties file.
            Runtime.setSourceProperty(Runtime.FILE_TEMPLATE_PATH, vTemplates);

            Runtime.init();
        }
        catch(Exception e)
        {
            org.apache.turbine.util.Log.error("Failed to set up Velocity templates:");
            org.apache.turbine.util.Log.error("Error setting velocity Configuration:");
            org.apache.turbine.util.Log.error(e);
            return;
        }
    }
}
