package org.apache.turbine.torque.transform;

/*
 * Copyright (c) 1997-2000 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * 4. The names "Apache JServ", "Apache JServ Servlet Engine", "Turbine",
 *    "Apache Turbine", "Turbine Project", "Apache Turbine Project" and
 *    "Java Apache Project" must not be used to endorse or promote products
 *    derived from this software without prior written permission.
 *
 * 5. Products derived from this software may not be called "Apache JServ"
 *    nor may "Apache" nor "Apache JServ" appear in their names without
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Group. For more information
 * on the Java Apache Project and the Apache JServ Servlet Engine project,
 * please see <http://java.apache.org/>.
 *
 */

import java.io.*;
import java.util.*;

import org.apache.turbine.torque.model.*;
import org.apache.turbine.torque.sql.*;

/**
 * A Class that converts an sql input file to an AppData
 * structure.  The class makes use of SQL Scanner to get
 * sql tokens and the parses these to create the AppData
 * class. SQLToAppData is in effect a simplified sql parser.
 *
 * 
 * @author <a href="mailto:leon@opticode.co.za>Leon Messerschmidt</a>
 * @version $Id: SQLToAppData.java,v 1.1 2000/11/07 23:39:16 jvanzyl Exp $
 */

public class SQLToAppData {

    private String sqlFile;
    private Vector tokens;
    private Token token;
    private AppData appData;
    private int count;

    /**
     * Create a new class without any input Reader.
     */
    public SQLToAppData()
    {
        this (null);
    }

    /**
     * Create a new class with an input Reader
     */
    public SQLToAppData (String sqlFile)
    {
        this.sqlFile = sqlFile;
    }


    /**
     * Get the current input sql file
     */
    public String getSqlFile()
    {
        return sqlFile;
    }

    /**
     * Set the current input sql file
     */
    public void setSqlFile (String sqlFile)
    {
        this.sqlFile = sqlFile;
    }

    /**
     * Move to the next token.  Throws an exception
     * if there is no more tokens available.
     */
    private void next() throws ParseException
    {
        if (count < tokens.size())
        {
            token = (Token) tokens.elementAt(count++);
        }
        else
        {
            throw new ParseException ("No More Tokens");
        }
    }

    /**
     * Creates an error condition and adds the line and
     * column number of the current token to the error
     * message.
     */
    private void err (String name) throws ParseException
    {
        throw new ParseException (name+"  [ line: "+token.getLine()+" col:"+token.getCol()+"]");
    }

    /**
     * Check if there is more tokens available for parsing.
     */
    private boolean hasTokens()
    {
        return count < tokens.size();
    }

    /**
     * Parses a CREATE sql command.
     */
    private void Create() throws ParseException
    {
        next();
        if (token.getStr().toUpperCase().equals("TABLE"))
            Create_Table();
    }

    /**
     * Parses a CREATE TABLE sql command
     */
    private void Create_Table() throws ParseException
    {
        next();
        String name = token.getStr();
        next();
        if (!token.getStr().equals("(")) err ("( expected");
        next();

        Table tbl = new Table (name);
        Create_Table_Column(tbl);
        while (token.getStr().equals(","))
        {
            next();
            Create_Table_Column(tbl);
        }

        if (!token.getStr().equals(")")) err (") expected");
        next();
        if (!token.getStr().equals(";")) err ("; expected");

        appData.addTable (tbl);
    }

    /**
     * Parses column information between the braces of a CREATE
     * TABLE () sql statement.
     */
    private void Create_Table_Column( Table tbl) throws ParseException
    {
        String name = token.getStr();

        if (token.getStr().toUpperCase().equals("PRIMARY"))
            Create_Table_Column_Primary (tbl);
        else if (token.getStr().toUpperCase().equals("FOREIGN"))
            Create_Table_Column_Foreign (tbl);
        else
            Create_Table_Column_Data (tbl);
    }

    /**
     * Parses PRIMARY KEY statement
     */
    private void Create_Table_Column_Primary (Table tbl) throws ParseException
    {
        next();
        if (!token.getStr().toUpperCase().equals("KEY"))
            err ("KEY expected");
        next();
        if (!token.getStr().toUpperCase().equals("("))
            err ("( expected");
        next();

        String colName = token.getStr();
        Column c = tbl.getColumn(colName);
        if (c == null) err ("Invalid column name: "+colName);
        c.setPrimaryKey(true);
        next();
        while (token.getStr().equals(","))
        {
            next();
            colName = token.getStr();
            c = tbl.getColumn(colName);
            if (c == null) err ("Invalid column name: "+colName);
            c.setPrimaryKey(true);
            next();
        }

        if (!token.getStr().toUpperCase().equals(")"))
            err (")expected");
        next();
    }

    /**
     * Parses FOREIGN KEY statement
     */
    private void Create_Table_Column_Foreign (Table tbl) throws ParseException
    {
        next();
        if (!token.getStr().toUpperCase().equals("KEY"))
            err ("KEY expected");
        next();
        if (!token.getStr().toUpperCase().equals("("))
            err ("( expected");
        next();

        ForeignKey fk = new ForeignKey();
        Vector localColumns = new Vector();
        tbl.addForeignKey(fk);

        String colName = token.getStr();
        localColumns.add(colName);
        next();
        while (token.getStr().equals(","))
        {
            next();
            colName = token.getStr();
            localColumns.add(colName);
            next();
        }
        if (!token.getStr().toUpperCase().equals(")"))
            err (") expected");

        next();

        if (!token.getStr().toUpperCase().equals("REFERENCES"))
            err ("REFERENCES expected");

        next();

        fk.setForeignTableName(token.getStr());

        next();

        if (token.getStr().toUpperCase().equals("("))
        {
            next();
            int i=0;
            fk.addReference((String)localColumns.get(i++),token.getStr());
            next();
            while (token.getStr().equals(","))
            {
                next();
                fk.addReference((String)localColumns.get(i++),token.getStr());
                next();
            }
            if (!token.getStr().toUpperCase().equals(")"))
                err (") expected");
            next();
        }

        while (token.getStr().toUpperCase().equals("ON"))
        {
            next();

            if (token.getStr().toUpperCase().equals("DELETE"))
            {
                fk.setOnDelete (Foreign_On_xxx());
            }
            else if (token.getStr().toUpperCase().equals("UPDATE"))
            {
                fk.setOnUpdate (Foreign_On_xxx());
            }
            else
            {
                err ("DELETE or UPDATE expected "+token.getStr()+" found.");
            }
        }
    }

    /**
     * Parse ON DELETE <XXX> and ON UPDATE <XXX> in foreign keys.
     */
    private int Foreign_On_xxx() throws ParseException
    {
        int result = ForeignKey.NONE;

        next();

        if (token.getStr().toUpperCase().equals("CASCADE"))
        {
            result = ForeignKey.CASCADE;
        }
        else if (token.getStr().toUpperCase().equals("RESTRICT"))
        {
            result = ForeignKey.RESTRICT;
        }
        else if (token.getStr().toUpperCase().equals("SET"))
        {
            next();
            if (!token.getStr().toUpperCase().equals("NULL"))
                err ("NULL expected");
            result = ForeignKey.SET_NULL;
        }
        else
        {
            err ("CASCADE or RESTRUCT or SET NULL expected");
        }

        next();

        return result;
    }


    /**
     * Parse the data definition of the column statement.
     */
    private void Create_Table_Column_Data ( Table tbl) throws ParseException
    {
        String name = token.getStr();
        next();
        String type = token.getStr();
        next();
        String size = null;
        String precision = null;

        if (token.getStr().toUpperCase().equals("("))
        {
            next();
            size = token.getStr();
            next();
            if (token.getStr().equals (","))
            {
                next();
                precision = token.getStr();
                next();
            }

            if (!token.getStr().equals (")"))
            {
                err (") expected");
            }
            next();

        }

        Column col = new Column (name);
        col.setTypeFromString (type,size);
        tbl.addColumn (col);

        while (!token.getStr().equals(",") && !token.getStr().equals(")"))
        {
            if (token.getStr().toUpperCase().equals("NOT"))
            {
                next();
                if (!token.getStr().toUpperCase().equals("NULL")) err ("NULL expected");
                col.setNotNull(true);
                next();
            }
            else if (token.getStr().toUpperCase().equals("PRIMARY"))
            {
                next();
                if (!token.getStr().toUpperCase().equals("KEY")) err ("KEY expected");
                col.setPrimaryKey(true);
                next();
            }
            else if (token.getStr().toUpperCase().equals("UNIQUE"))
            {
                col.setUnique(true);
                next();
            }
            else if (token.getStr().toUpperCase().equals("NULL"))
            {
                col.setNotNull(false);
                next();
            }
            else err ("Unknown identifier "+token.getStr());


        }
    }

    /**
     * Execute the parser.
     */
    public AppData execute() throws IOException, ParseException
    {
        count = 0;
        appData = new AppData();

        FileReader fr = new FileReader (sqlFile);
        BufferedReader br = new BufferedReader (fr);
        SQLScanner scanner = new SQLScanner(br);

        tokens = scanner.scan();

        br.close();

        while (hasTokens())
        {
            next();
            if (token.getStr().toUpperCase().equals("CREATE"))
            {
                Create();
            }
            else
            {
                err ("CREATE expected");
            }

        }

        return appData;
    }


    /**
     * Just 4 testing.
     */
    public static void main (String args[]) throws Exception
    {
        SQLToAppData s2a = new SQLToAppData (args[0]);
        AppData ad = s2a.execute();
        System.out.println (ad);
    }
}
