package org.apache.turbine.util;

/*
 * Copyright (c) 1997-2000 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * 4. The names "Apache JServ", "Apache JServ Servlet Engine", "Turbine",
 *    "Apache Turbine", "Turbine Project", "Apache Turbine Project" and
 *    "Java Apache Project" must not be used to endorse or promote products
 *    derived from this software without prior written permission.
 *
 * 5. Products derived from this software may not be called "Apache JServ"
 *    nor may "Apache" nor "Apache JServ" appear in their names without
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Group. For more information
 * on the Java Apache Project and the Apache JServ Servlet Engine project,
 * please see <http://java.apache.org/>.
 *
 */

// Java Core Classes
import java.io.*;
import java.util.*;

// Java Servlet Classes
import javax.servlet.*;

// Turbine Logging Facility
import org.apache.turbine.util.Log;

/**
 * A class used for initalization of Turbine without a servlet container.
 *
 * If you need to use Turbine outside of a servlet container, you can
 * use this class for initalization of the Turbine servlet.<br>
 *
 * <pre>
 *   TurbineConfig config = new TurbineConfig(".","/conf/TurbineResources.properties");
 *   Turbine turbine = new Turbine();
 *   turbine.init(config);</pre>
 * 
 * All paths referenced in TurbineResources.properties and the path to
 * the properties file itself (the second argument) will be resolved 
 * relative to the directory given as the first argument of the constructor,
 * here - the directory where application was started. Don't worry about 
 * discarding the references to objects created above. They are not needed, 
 * once Turbine is initialized.
 * 
 * @author Rafal Krzewski, krzewski@e-point.pl
 */
public class TurbineConfig implements ServletConfig, ServletContext {

    /** Enables output of debug messages. (compile time option) */
    private final static boolean debug = false;
    
    /** Filenames are looked up in those directories. */
    private File[] roots;    

    /** Initialization parameters */
    private Map initParams;

    /**
     * Constructs a new TurbineConfig.
     *
     * This is the general form of the costructor. You can provide
     * a list of paths to search, and a name-value map of init
     * parameters.
     *
     * @param paths the paths for file lookup.
     * @param initParams initialization parameters.
     */
    public TurbineConfig( String[] paths, Map initParams ) 
    {
        roots = new File[paths.length];
        for(int i=0; i<paths.length; i++) 
        {
            roots[i] = new File(paths[i]);
        }
        this.initParams = initParams;
    }

    /**
     * Constructs a TurbineConfig.
     *
     * This is a specialized constructor that allows to configure
     * Turbine easiliy in the common setups.
     *
     * @param path the path for file lookup.
     * @param properties the relative path to TurbineResources.properties file
     */
    public TurbineConfig( String path, String properties ) 
    {
        roots = new File[] { new File(path) };
        initParams = new HashMap(1);
        initParams.put(org.apache.turbine.Turbine.INIT_PROPS_KEY, properties);
    }
    
    /**
     * Retrns a reference to the object cast onto ServletContext type.
     *
     * @return a ServletContext reference
     */
    public ServletContext getServletContext() {
        return this;
    }
    
    /**
     * Translates a relative path into an absolute one.
     *
     * @param path a relative path
     * @return an absolute path
     */
    public String getRealPath( String path ) 
    {
        if(debug) 
        {
            Log.note("getRealPath: "+path);
        }

        File f = new File("");
        for(int i=0; !f.exists() && i<roots.length; i++) 
        {
            f = new File(roots[i], path);
        }

        if(debug) 
        { 
            if(f.exists()) 
            {
                Log.note("found "+f.getPath());
                return f.getPath();
            } else 
            {
                Log.note("not found");   
                return null;
            }
        }
        else // !debug
        {
            if(f.exists()) 
            {
                return f.getPath();
            } 
            else 
            {
                return null;
            }
        }
    }
    
    /**
     * Retrieves an initialization parameter.
     * 
     * @param name the name of the parameter.
     * @return the value of the parameter.
     */
    public String getInitParameter(String name) 
    {
        return (String)initParams.get(name);
    }
    
    /**
     * Retrieves an Enumeration of initailzation parameter names.
     *
     * @return an Enumeration of initailzation parameter names.
     */
    public Enumeration getInitParameterNames() 
    {
        return new Vector(initParams.keySet()).elements();
    }

    /**
     * Returns the servlet name.
     *
     * Fixed value "Turbine" is returned.
     *
     * @return the servlet name.
     */
    public String getServletName() 
    {
        return "Turbine";
    }

    /**
     * Logs an error message.
     *
     * @param e an Exception.
     * @param m a message.
     */
    public void log(Exception e, String m) 
    {
        // cannot use Turbine logging yet.
        System.out.println(m);
        e.printStackTrace();
    }
    
    /**
     * Logs a message.
     *
     * @param m a message.
     */
    public void log(String m) 
    {
        // cannot use Turbine logging yet.
        System.out.println(m);
    }

    /**
     * Logs an error message.
     *
     * @param t a Throwable object.
     * @param m a message.
     */
    public void log( String m, Throwable t ) 
    {
        // cannot use Turbine logging yet.
        System.out.println(m);
        t.printStackTrace();
    }

    // Unimplemented methods follow
    
    /**
     * Not implemented.
     *
     * A method in ServletConfig or ServletContext interface
     * that is not implemented and will throw <code>UnsuportedOperationException</code>
     * upon invocation
     */
    public Object getAttribute(String s) 
    {
        throw new UnsupportedOperationException();
    }

    /**
     * Not implemented.
     *
     * A method in ServletConfig or ServletContext interface
     * that is not implemented and will throw <code>UnsuportedOperationException</code>
     * upon invocation
     */
    public Enumeration getAttributeNames() 
    {
        throw new UnsupportedOperationException();
    }

    /**
     * Not implemented.
     *
     * A method in ServletConfig or ServletContext interface
     * that is not implemented and will throw <code>UnsuportedOperationException</code>
     * upon invocation
     */
    public ServletContext getContext(String s) 
    {
        throw new UnsupportedOperationException();
    }

    /**
     * Not implemented.
     *
     * A method in ServletConfig or ServletContext interface
     * that is not implemented and will throw <code>UnsuportedOperationException</code>
     * upon invocation
     */
    public int getMajorVersion() 
    {
        throw new UnsupportedOperationException();
    }

    /**
     * Not implemented.
     *
     * A method in ServletConfig or ServletContext interface
     * that is not implemented and will throw <code>UnsuportedOperationException</code>
     * upon invocation
     */
    public String getMimeType(String s) 
    {
        throw new UnsupportedOperationException();
    }

    /**
     * Not implemented.
     *
     * A method in ServletConfig or ServletContext interface
     * that is not implemented and will throw <code>UnsuportedOperationException</code>
     * upon invocation
     */
    public int getMinorVersion() 
    {
        throw new UnsupportedOperationException();
    }

    /**
     * Not implemented.
     *
     * A method in ServletConfig or ServletContext interface
     * that is not implemented and will throw <code>UnsuportedOperationException</code>
     * upon invocation
     */
    public RequestDispatcher getNamedDispatcher( String s) 
    {
        throw new UnsupportedOperationException();
    }

    /**
     * Not implemented.
     *
     * A method in ServletConfig or ServletContext interface
     * that is not implemented and will throw <code>UnsuportedOperationException</code>
     * upon invocation
     */
    public RequestDispatcher getRequestDispatcher( String s ) 
    {
        throw new UnsupportedOperationException();
    }

    /**
     * Not implemented.
     *
     * A method in ServletConfig or ServletContext interface
     * that is not implemented and will throw <code>UnsuportedOperationException</code>
     * upon invocation
     */
    public java.net.URL getResource( String s ) 
    {
        throw new UnsupportedOperationException();
    }

    /**
     * Not implemented.
     *
     * A method in ServletConfig or ServletContext interface
     * that is not implemented and will throw <code>UnsuportedOperationException</code>
     * upon invocation
     */
    public InputStream getResourceAsStream( String s ) 
    {
        throw new UnsupportedOperationException();
    }

    /**
     * Not implemented.
     *
     * A method in ServletConfig or ServletContext interface
     * that is not implemented and will throw <code>UnsuportedOperationException</code>
     * upon invocation
     */
    public String getServerInfo() 
    {
        throw new UnsupportedOperationException();
    }
    
    /**
     * Not implemented.
     *
     * A method in ServletConfig or ServletContext interface
     * that is not implemented and will throw <code>UnsuportedOperationException</code>
     * upon invocation
     */
    public Servlet getServlet( String s) 
    {
        throw new UnsupportedOperationException();
    }

    /**
     * Not implemented.
     *
     * A method in ServletConfig or ServletContext interface
     * that is not implemented and will throw <code>UnsuportedOperationException</code>
     * upon invocation
     */
    public Enumeration getServletNames() 
    {
        throw new UnsupportedOperationException();
    }

    /**
     * Not implemented.
     *
     * A method in ServletConfig or ServletContext interface
     * that is not implemented and will throw <code>UnsuportedOperationException</code>
     * upon invocation
     */
    public Enumeration getServlets() 
    {
        throw new UnsupportedOperationException();
    }

    /**
     * Not implemented.
     *
     * A method in ServletConfig or ServletContext interface
     * that is not implemented and will throw <code>UnsuportedOperationException</code>
     * upon invocation
     */
    public void removeAttribute( String s ) 
    {
        throw new UnsupportedOperationException();
    }

    /**
     * Not implemented.
     *
     * A method in ServletConfig or ServletContext interface
     * that is not implemented and will throw <code>UnsuportedOperationException</code>
     * upon invocation
     */
    public void setAttribute( String s, Object o ) 
    {
        throw new UnsupportedOperationException();
    }
}
