package org.apache.turbine.util.db.pool;

/*
 * Copyright (c) 1997-2000 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * 4. The names "Apache JServ", "Apache JServ Servlet Engine", "Turbine",
 *    "Apache Turbine", "Turbine Project", "Apache Turbine Project" and
 *    "Java Apache Project" must not be used to endorse or promote products
 *    derived from this software without prior written permission.
 *
 * 5. Products derived from this software may not be called "Apache JServ"
 *    nor may "Apache" nor "Apache JServ" appear in their names without
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Group. For more information
 * on the Java Apache Project and the Apache JServ Servlet Engine project,
 * please see <http://java.apache.org/>.
 *
 */

// Java Core Classes
import java.sql.*;
import java.util.*;

// Turbine stuff.
import org.apache.turbine.util.Log;

/**
 * This class wraps the JDBC <code>Connection</code> class, providing
 * built-in exception logging to stderr and a timestamp which
 * indicates how old the <code>Connection</code> is.
 * <code>DBConnection</code>'s timestamp is used by the
 * <code>ConnectionPool</code> to expire connections when they get too
 * old.
 *
 * @author <a href="mailto:frank.kim@clearink.com">Frank Y. Kim</a>
 * @author <a href="mailto:john.mcnally@clearink.com">John D. McNally</a>
 * @author <a href="mailto:dlr@collab.net">Daniel L. Rall</a>
 * @author <a href="mailto:magnus@handtolvur.is">Magns r Torfason</a>
 * @version $Id: DBConnection.java,v 1.6 2000/10/21 20:42:33 dlr Exp $
 */
public class DBConnection
{
    /**
     * The ConnectionPool that this DBConnection came from.
     * A null means that the DBConnection either does not
     * belong to any pool, or that it is currently inside it's pool
     */
    private ConnectionPool pool = null;

    /**
     * The JDBC database connection.
     */
    private Connection connection = null;

    /**
     * The URL of this connection.
     */
    private String url = null;

    /**
     * The user name for this connection.
     */
    private String username = null;

    /**
     * The time in milliseconds at which the connection was
     * created.
     */
    private long timestamp;

    /**
     * Creates a Turbine <code>DBConnection</code> specifying
     * nothing but a single database Connection.
     *
     * @param connection The JDBC connection to wrap.
     */
    protected DBConnection(Connection connection)
    {
        this(connection, null, null, null);
    }

    /**
     * Creates a Turbine <code>DBConnection</code> with the specified
     * attributes.
     *
     * @param connection The JDBC connection to wrap.
     * @param url        The URL we're connecting to.
     */
    protected DBConnection(Connection connection, String url)
    {
        this(connection, url, null, null);
    }

    /**
     * Creates a Turbine <code>DBConnection</code> with the specified
     * attributes.
     *
     * @param connection The JDBC connection to wrap.
     * @param url        The URL we're connecting to.
     * @param username   The user name we are connecting as
     */
    protected DBConnection(Connection connection, String url, String username)
    {
        this(connection, url, username, null);
    }

    /**
     * Creates a Turbine <code>DBConnection</code> that is part of
     * a pool.
     *
     * @param connection The JDBC connection to wrap.
     * @param url        The URL we're connecting to.
     * @param username   The user name we are connecting as
     * @param pool       The ConnectionPool that this DBConnection belongs to
     */
    protected DBConnection(Connection connection, String url, String username, 
                           ConnectionPool pool)
    {
        this.connection = connection;
        this.url = url;
        this.username = username;
        this.pool = pool;
        this.timestamp = System.currentTimeMillis();
    }

    /**
     * The finalizer helps prevent <code>ConnectionPool</code> leakage.
     */
    protected void finalize()
        throws Throwable
    {
        if (pool != null)
        {
            // If this DBConnection object is finalized while linked
            // to a ConnectionPool, it means that it was taken from a pool
            // and not returned.  We log this fact, close the underlying
            // Connection, and return it to the ConnectionPool.
            Log.warn( "A DBConnection was finalized, without being returned "
                      + "to the ConnectionPool it belonged to" );

            // Closing the Connection ensures that if anyone tries to use it,
            // an error will occur.
            connection.close();

            // Releasing a new DBConnection object will prevent leaks
            // from the pool
            pool.releaseConnection(new DBConnection(connection, url, username, 
                                                    pool));
        }
    }


    /**
     * Links this DBConnection with a ConnectionPool.
     *
     * @param The pool to link to
     */
    protected void link(ConnectionPool pool)
    {
        if (pool == null)
        {
            throw new NullPointerException("Cannot link to a null pool");
        }
        this.pool = pool;
    }

    /**
     * Unlink the DBConnection from it's pool.
     * @param The pool to unlink from
     * @exception Thrown if someone tries to unlink from another pool
     */
    protected void unlink(ConnectionPool pool) throws Exception
    {
        if (this.pool != pool && pool != null)
        {
            throw new NullPointerException
                ("Trying to unlink from the wrong pool");
        }
        this.pool = null;
    }

    /**
     * Returns the pool this DBConnection came from, or null if
     * it is not linked to any pool
     *
     * @return The pool this connection came from.
     */
    public ConnectionPool getPool()
    {
        return pool;
    }

    /**
     * Returns a JDBC connection.
     *
     * @return The database connection.
     */
    public Connection getConnection()
    {
        return connection;
    }

    /**
     * Returns a long representing the time this connection was
     * created.
     *
     * @return The time in milliseconds that this connection was
     * created.
     */
    public long getTimestamp()
    {
        return timestamp;
    }

    /**
     * Returns the connection URL.
     *
     * @return A String with the connection URL.
     */
    public String getUrl()
    {
        return url;
    }

    /**
     * Returns the connection user name.
     *
     * @return A String with the connection username.
     */
    public String getUsername()
    {
        return username;
    }

    /**
     * Commit the connection.
     */
    public void commit()
    {
        try
        {
            connection.commit();
        }
        catch(SQLException e)
        {
            e.printStackTrace();
            Log.error(e);
        }
    }

    /**
     * Roll back the connection.
     */
    public void rollback()
    {
        try
        {
            connection.rollback();
        }
        catch(SQLException e)
        {
            e.printStackTrace();
            Log.error(e);
        }
    }

    /**
     * Set the autocommit flag for the connection.
     *
     * @param b True if autocommit should be set to true.
     */
    public void setAutoCommit(boolean b)
    {
        try
        {
            connection.setAutoCommit(b);
        }
        catch(SQLException e)
        {
            e.printStackTrace();
            Log.error(e);
        }
    }

    /**
     * Create a Java SQL statement for this connection.
     *
     * @return A new <code>Statement</code>.
     */
    public java.sql.Statement createStatement()
    {
        java.sql.Statement stmt = null;
        try
        {
            stmt = connection.createStatement();
        }
        catch (SQLException e)
        {
            e.printStackTrace();
            Log.error(e);
        }
        return stmt;
    }

    /**
     * Create a prepared Java SQL statement for this connection.
     *
     * @param sql The SQL statement to prepare.
     * @return A new <code>PreparedStatement</code>.
     */
    public java.sql.PreparedStatement prepareStatement(String sql)
    {
        java.sql.PreparedStatement stmt = null;
        try
        {
            stmt = connection.prepareStatement(sql);
        }
        catch (SQLException e)
        {
            e.printStackTrace();
            Log.error(e);
        }
        return stmt;
    }

    /**
     * Force the close of this database connection.
     *
     * @param dbConn The connection to close.
     */
    public void close()
    {
        Connection conn = getConnection();
        try
        {
            if ( conn != null && !conn.isClosed() )
            {
                conn.close();
            }
        }
        catch (Exception ignored)
        {
            Log.warn("Couldn't close database connection");
        }
    }
}
