package org.apache.turbine.util.security;

/*
 * Copyright (c) 1997-2000 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * 4. The names "Apache JServ", "Apache JServ Servlet Engine", "Turbine",
 *    "Apache Turbine", "Turbine Project", "Apache Turbine Project" and
 *    "Java Apache Project" must not be used to endorse or promote products
 *    derived from this software without prior written permission.
 *
 * 5. Products derived from this software may not be called "Apache JServ"
 *    nor may "Apache" nor "Apache JServ" appear in their names without
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Group. For more information
 * on the Java Apache Project and the Apache JServ Servlet Engine project,
 * please see <http://java.apache.org/>.
 *
 */

// Java classes
import java.util.Map;
import java.util.Set;
import java.io.Serializable;

// Turbine security classes
import org.apache.turbine.services.security.TurbineSecurity;
import org.apache.turbine.om.security.*;

/**
 * This is a control class that makes it easy to find out if a
 * particular User has a given Permission.  It also determines if a
 * User has a a particular Role.
 *
 * @author <a href="mailto:john.mcnally@clearink.com">John D. McNally</a>
 * @author <a href="mailto:bmclaugh@algx.net">Brett McLaughlin</a>
 * @author <a href="mailto:greg@shwoop.com">Greg Ritter</a>
 * @author <a href="mailto:Rafal.Krzewski@e-point.pl">Rafal Krzewski</a>
 * @version $Id: AccessControlList.java,v 1.3 2000/11/19 02:33:58 rafal Exp $
 */
public class AccessControlList implements Serializable
{
    /** The sets of roles that the user has in different groups */
    private Map roleSets;

    /** The sets of permissions that the user has in different groups */
    private Map permissionSets;

    public static java.lang.String SESSION_KEY = "turbine.AccessControlList";

    /**
     * Constructs a new AccessControlList.
     *
     * This class follows 'immutable' pattern - it's objects can't be modified
     * once they are created. This means that the permissions the users have are
     * in effect form the moment they log in to the moment they log out, and 
     * changes made to the security settings in that time are not reflected
     * in the state of this object. If you need to reset an user's permissions
     * you need to invalidate his session. <br>
     * The objects that constructs an AccessControlList must supply hashtables 
     * of role/permission sets keyed with group objects. <br>
     *
     * @param roleSets a hashtable containing RoleSet objects keyed with Group objects
     * @param permissionSets a hashtable containing PermissionSet objects keyed with Group objects
     */
    public AccessControlList( Map roleSets, Map permissionSets )
    {
        this.roleSets = roleSets;
        this.permissionSets = permissionSets;
    }

    /**
     * Retrieves a set of Roles an user is assigned in a Group.
     *
     * @param group the Group 
     * @return the set of Roles this user has within the Group.
     */
    public RoleSet getRoles( Group group )
    {
        if(group == null)
            return null;
        return (RoleSet)roleSets.get(group);
    }

    /**
     * Retrieves a set of Roles an user is assigned in the global Group.
     *
     * @param group the Group 
     * @return the set of Roles this user has within the global Group.
     */
    public RoleSet getRoles()
    {
        return getRoles(Group.getGlobalGroup());
    }

    /**
     * Retrieves a set of Permissions an user is assigned in a Group.
     *
     * @param group the Group 
     * @return the set of Permissions this user has within the Group.
     */
    public PermissionSet getPermissions( Group group )
    {
        if(group == null)
            return null;
        return (PermissionSet)permissionSets.get(group);
    }

    /**
     * Retrieves a set of Permissions an user is assigned in the global Group.
     *
     * @param group the Group 
     * @return the set of Permissions this user has within the global Group.
     */
    public PermissionSet getPermissions()
    {
        return getPermissions(Group.getGlobalGroup());
    }

    /**
     * Checks if the user is assigned a specific Role in the Group.
     * 
     * @param role the Role
     * @param group the Group
     * @return <code>true</code> if the user is assigned the Role in the Group.
     */
    public boolean hasRole( Role role, Group group )
    {
        RoleSet set = getRoles(group);
        if(set == null || role == null)
            return false;
        return set.contains(role);
    }

    /**
     * Checks if the user is assigned a specific Role in the Group.
     * 
     * @param role the Role
     * @param group the Group
     * @return <code>true</code> if the user is assigned the Role in the Group.
     */
    public boolean hasRole( String role, String group )
    {
        try
        {
            return hasRole(TurbineSecurity.getRole(role), TurbineSecurity.getGroup(group));
        }
        catch(Exception e)
        {
            return false;
        }
    }

    /**
     * Checks if the user is assigned a specific Role in the global Group.
     * 
     * @param role the Role
     * @param group the Group
     * @return <code>true</code> if the user is assigned the Role in the global Group.
     */
    public boolean hasRole( Role role )
    {
        return hasRole(role, Group.getGlobalGroup());
    }

    /**
     * Checks if the user is assigned a specific Role in the global Group.
     * 
     * @param role the Role
     * @param group the Group
     * @return <code>true</code> if the user is assigned the Role in the global Group.
     */
    public boolean hasRole( String role )
    {
        try
        {
            return hasRole(TurbineSecurity.getRole(role));
        }
        catch(Exception e)
        {
            return false;
        }
    }

    /**
     * Checks if the user is assigned a specific Permission in the Group.
     * 
     * @param permission the Permission
     * @param group the Group
     * @return <code>true</code> if the user is assigned the Permission in the Group.
     */
    public boolean hasPermission( Permission permission, Group group )
    {
        PermissionSet set = getPermissions(group);
        if(set == null || permission == null)
            return false;
        return set.contains(permission);
    }

    /**
     * Checks if the user is assigned a specific Permission in the Group.
     * 
     * @param permission the Permission
     * @param group the Group
     * @return <code>true</code> if the user is assigned the Permission in the Group.
     */
    public boolean hasPermission( String permission, String group )
    {
        try
        {
            return hasPermission(TurbineSecurity.getPermission(permission), 
                                 TurbineSecurity.getGroup(group));
        }
        catch(Exception e)
        {
            return false;
        }
    }

    /**
     * Checks if the user is assigned a specific Permission in the global Group.
     * 
     * @param permission the Permission
     * @param group the Group
     * @return <code>true</code> if the user is assigned the Permission in the global Group.
     */
    public boolean hasPermission( Permission permission )
    {
        return hasPermission(permission, Group.getGlobalGroup());
    }

    /**
     * Checks if the user is assigned a specific Permission in the global Group.
     * 
     * @param permission the Permission
     * @param group the Group
     * @return <code>true</code> if the user is assigned the Permission in the global Group.
     */
    public boolean hasPermission( String permission )
    {
        try
        {
            return hasPermission(TurbineSecurity.getPermission(permission));
        }
        catch(Exception e)
        {
            return false;
        }
    }

    /**
     * Returns all groups definded in the system.
     *
     * This is useful for debugging, when you want to display all roles
     * and permissions an user is assingned. This method is needed
     * because you can't call static methods of TurbineSecurity class
     * from within WebMacro/Velocity template
     */
    public Group[] getAllGroups()
    {
        try 
        {
            return TurbineSecurity.getAllGroups().getGroupsArray();
        }
        catch(TurbineSecurityException e) 
        {
            return new Group[0];
        }
    }
}
