package org.apache.turbine.util.security;

/*
 * Copyright (c) 1997-2000 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * 4. The names "Apache JServ", "Apache JServ Servlet Engine", "Turbine",
 *    "Apache Turbine", "Turbine Project", "Apache Turbine Project" and
 *    "Java Apache Project" must not be used to endorse or promote products
 *    derived from this software without prior written permission.
 *
 * 5. Products derived from this software may not be called "Apache JServ"
 *    nor may "Apache" nor "Apache JServ" appear in their names without
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Group. For more information
 * on the Java Apache Project and the Apache JServ Servlet Engine project,
 * please see <http://java.apache.org/>.
 *
 */

// Java Core Classes
import java.util.*;
import java.io.*;

// Security related objects
import org.apache.turbine.om.security.*;

/**
 * This class represents a set of Permissions.  It makes it easy to
 * build a UI that would allow someone to add a group of Permissions
 * to a Role.  It wraps a TreeSet object to enforce that only
 * Permission objects are allowed in the set and only relevant methods
 * are available.  TreeSet's contain only unique Objects (no
 * duplicates).
 *
 * @author <a href="mailto:john.mcnally@clearink.com">John D. McNally</a>
 * @author <a href="mailto:bmclaugh@algx.net">Brett McLaughlin</a>
 * @version $Id: PermissionSet.java,v 1.3 2000/11/19 02:33:58 rafal Exp $
 */
public class PermissionSet implements Serializable
{
    /** Set to hold the Permission Set */
    private TreeSet set;

    /**
     * Constructs an empty PermissionSet
     */
    public PermissionSet()
    {
        set = new TreeSet();
    }
    
    /**
     * Constructs a new PermissionSet with specifed contents.
     * 
     * If the given collection contains multiple objects that are
     * identical WRT equals() method, some objects will be overwriten.
     *
     * @param permissions A collection of permissions to be contained in the set.
     */
    public PermissionSet(Collection permissions)
    {
        this();
        add(permissions);
    }

    /**
     * Adds a Permission to this PermissionSet.
     *
     * @param group A Permission.
     * @return True if Permission was added; false if PermissionSet
     * already contained the Permission.
     */
    public boolean add(Permission group)
    {
        return set.add( (Object)group );
    }

    /**
     * Adds the Permissions in a Collection to this PermissionSet.
     *
     * @param permissionSet A Collection of Permissions.
     * @return True if this PermissionSet changed as a result; false
     * if no change to this PermissionSet occurred (this PermissionSet
     * already contained all members of the added PermissionSet).
     */
    public boolean add(Collection permissions)
    {
        return set.addAll(permissions);
    }

    /**
     * Adds the Permissions in another PermissionSet to this
     * PermissionSet.
     *
     * @param groupSet A PermissionSet.
     * @return True if this PermissionSet changed as a result; false
     * if no change to this PermissionSet occurred (this PermissionSet
     * already contained all members of the added PermissionSet).
     */
    public boolean add(PermissionSet groupSet)
    {
        return set.addAll(  (Collection) groupSet.set );
    }

    /**
     * Removes a Permission from this PermissionSet.
     *
     * @param group A Permission.
     * @return True if this PermissionSet contained the Permission
     * before it was removed.
     */
    public boolean remove(Permission group)
    {
        return set.remove( (Object)group );
    }

    /**
     * Removes all Permissions from this PermissionSet.
     */
    public void clear()
    {
        set.clear();
    }

    /**
     * Checks whether this PermissionSet contains a Permission.
     *
     * @param group A Permission.
     * @return True if this PermissionSet contains the Permission,
     * false otherwise.
     */
    public boolean contains(Permission group)
    {
        return set.contains( (Object)group );
    }

    /**
     * Compares by name a Permission with the Permissions contained in
     * this PermissionSet.
     *
     * @param groupName Name of Permission.
     * @return True if argument matched a Permission in this
     * PermissionSet; false if no match.
     */
    public boolean contains(String groupName)
    {
        Iterator iter = set.iterator();
        while ( iter.hasNext() )
        {
            Permission group = (Permission)iter.next();
            if ( groupName != null  &&
                 groupName.equals( group.getName() ) )
            {
                return true;
            }
        }
        return false;
    }

    /**
     * Returns a Permission with the given name, if it is contained in
     * this PermissionSet.
     *
     * @param groupName Name of Permission.
     * @return Permission if argument matched a Permission in this
     * PermissionSet; null if no match.
     */
    public Permission getPermission(String groupName)
    {
        Iterator iter = set.iterator();
        while ( iter.hasNext() )
        {
            Permission group = (Permission)iter.next();
            if ( groupName != null  &&
                 groupName.equals( group.getName() ) )
            {
                return group;
            }
        }
        return null;
    }

    /**
     * Returns an Permissions[] of Permissions in this PermissionSet.
     *
     * @return A Permission[].
     */
    public Permission[] getPermissionsArray()
    {
        return (Permission[])set.toArray(new Permission[0]);
    }

    /**
     * Returns an Iterator for Permissions in this PermissionSet.
     */
    public Iterator elements()
    {
        return set.iterator();
    }

    /**
     * Returns size (cardinality) of this set.
     *
     * @return The cardinality of this PermissionSet.
     */
    public int size()
    {
        return set.size();
    }
}
